@@ -1,6 +1,6 @@
-print.confLCCR <-
-function(x, ...){
+print.confLCCR = function(x, ...){
 
+#---- print output of confLCCR ---
   cat("\nConfidence interval for the populations size based on latent class models")
   cat("\nfor capture-recapture data\n")
   cat("\n")
