@@ -1,6 +1,6 @@
-summary.confLCCR <-
-function(object, ...){
+summary.confLCCR = function(object, ...){
 
+#---- summarize output of confLCCR ---
   cat("\nConfidence interval for the populations size based on latent class models")
   cat("\nfor capture-recapture data\n")
   cat("\n")
